#include "qtmightability.hpp"
#include "ui_qtmightability.h"

#include<fstream>

#include <Util-pkg.h>
#include <P3d-pkg.h>
#include <Planner-pkg.h>
#include <Localpath-pkg.h>
#include <Collision-pkg.h>
#include <Graphic-pkg.h>

#include <libmove3d/hri/hri.h>

using namespace std;

extern int SHOW_TASKABILITIES;
extern show_taskability_params curr_params_for_show_taskability;//It includes params for manipulability also

extern std::vector<std::pair<std::string, HRI_TASK_TYPE> > HRI_task_NAME_Enum_map;
extern std::vector<std::pair<std::string, HRI_TASK_AGENT> > HRI_agent_NAME_Enum_map;

extern HRI_TASK_TYPE CURRENT_HRI_MANIPULATION_TASK;
extern char CURRENT_OBJECT_TO_MANIPULATE[50];
extern HRI_TASK_AGENT CURRENT_TASK_PERFORMED_BY;
extern HRI_TASK_AGENT CURRENT_TASK_PERFORMED_FOR;
extern int TASK_IS_FOR_PROACTIVE_BEHAVIOR;

// default objects and agents:
QString defaultCentralObject("LOW_TABLE_LARGE");
QString defaultObject("GREY_TAPE");
QString defaultAgentFor("HUMAN1_MA");
QString defaultperformer("PR2_MA");
QString defaultTask("PICK_MAKE_OBJECT_ACCESSIBLE");




qtMightability::qtMightability(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::qtMightability)
{
    ui->setupUi(this);


    ui->groupBoxTaskSpec->setDisabled(true);
    ui->groupBoxPlaning->setDisabled(true);
    ui->groupBoxExec->setDisabled(true);
    ui->groupBoxGTP->setDisabled(true);
    ui->groupBoxComputedTasks->setDisabled(true);
    ui->groupBoxTaskability->setDisabled(true);

    for(int i =0;i<XYZ_ENV->nr;i++)
    {
        if( XYZ_ENV->robot[i]->njoints == 1 )
        {
            if(XYZ_ENV->robot[i]->joints[1]->type == P3D_FREEFLYER || XYZ_ENV->robot[i]->joints[1]->type == P3D_QUADROTOR)
            {
                QString FFname(XYZ_ENV->robot[i]->name);
                FreeFlyers.push_back(FFname);
            }
        }
        else if ( XYZ_ENV->robot[i]->njoints >= 10)
        {
            QString Aname(XYZ_ENV->robot[i]->name);
            if (Aname.contains("ROBOT") || Aname.contains("HUMAN"))
            {
                Agents.push_back(Aname);
            }
        }
    }

    //fill combobox

    ui->comboBoxCentralObject->clear();
    int Cid = 0;
    for (unsigned int i = 0; i <FreeFlyers.size() ; i++)
    {

        if (FreeFlyers.at(i).contains("TABLE") || FreeFlyers.at(i).contains("SHELF"))
        {
            ui->comboBoxCentralObject->addItem(FreeFlyers.at(i));

            if (FreeFlyers.at(i).contains(defaultCentralObject))
            {
                ui->comboBoxCentralObject->setCurrentIndex(Cid);
            }
            Cid++;
        }
        else
        {
            ui->comboBoxObject->addItem(FreeFlyers.at(i));
            if (FreeFlyers.at(i).contains(defaultObject))
            {
                ui->comboBoxObject->setCurrentIndex(i);
            }
        }
    }




}

qtMightability::~qtMightability()
{
    delete ui;
}

void qtMightability::changeEvent(QEvent *e)
{
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void qtMightability::initMightabilities()
{

    GLOBAL_AGENTS = hri_create_agents();
    hri_assign_source_agent((char*)"JIDOKUKA", GLOBAL_AGENTS);

    GLOBAL_ENTITIES = hri_create_entities();
    hri_link_agents_with_entities(GLOBAL_ENTITIES, GLOBAL_AGENTS);
    hri_initialize_all_agents_knowledge(GLOBAL_ENTITIES, GLOBAL_AGENTS);

#ifdef USE_MIGHTABILITY_MAPS
    init_agents_for_MA_and_ASA();
#else
    cout<< "Flag USE_MIGHTABILITY_MAPS not on" <<endl;
#endif


    char threshold_file_path[150];
    char *home_dir=getenv("HOME");
    if (ENV.getBool(Env::printMightabilities))
        cout << " Home directory is" << home_dir << endl;

    strcpy(threshold_file_path,home_dir);
    char threshold_file_name[50]="/Human_State_Analysis_Thresholds.txt";
    strcat(threshold_file_path,threshold_file_name);

    //"/home/akpandey/Human_State_Analysis_Thresholds.txt";

    ifstream file(threshold_file_path);
    if (!file)
    {
        cout << "the file /Human_State_Analysis_Thresholds.txt does not exist in "
             << home_dir << "please added and launch init again" << endl;
        return;
    }
    prepare_for_Agent_State_Analysis(threshold_file_path);


    char MM_around_object[50]="LOW_TABLE_LARGE"; //"TABLE_4"
    strcpy(MM_around_object, ui->comboBoxCentralObject->currentText().toStdString().c_str());
#ifdef USE_MIGHTABILITY_MAPS
    int MA_init_res=Create_and_init_Mightability_Maps(MM_around_object);
#else
    cout<< "Flag USE_MIGHTABILITY_MAPS not on" <<endl;
#endif
}

void qtMightability::initGui()
{

//    ui->comboBoxTask->addItem(QString::number(HRI_task_NAME_Enum_map.size()));
    int HRid = 0;
    for(unsigned int ii = 0; ii < HRI_task_NAME_Enum_map.size(); ii++)
    {
        ui->comboBoxTask->addItem(QString::fromStdString(HRI_task_NAME_Enum_map.at(ii).first));
        if (QString::fromStdString(HRI_task_NAME_Enum_map.at(ii).first).contains(defaultTask))
        {
            ui->comboBoxTask->setCurrentIndex(HRid);
        }
        HRid++;
    }

    for (unsigned int i = 0; i <HRI_agent_NAME_Enum_map.size() ; i++)
    {
        QString str = QString::fromStdString(HRI_agent_NAME_Enum_map.at(i).first);
        ui->comboBoxAgentFor->addItem(str);
        if (str.contains(defaultAgentFor))
        {
            ui->comboBoxAgentFor->setCurrentIndex(i);
        }
        ui->comboBoxPerformer->addItem(str);
        if (str.contains(defaultperformer))
        {
            ui->comboBoxPerformer->setCurrentIndex(i);
        }
    }

    //enable gui
    ui->groupBoxTaskSpec->setDisabled(false);
    ui->groupBoxPlaning->setDisabled(false);
    ui->groupBoxExec->setDisabled(false);
    ui->groupBoxGTP->setDisabled(false);
    ui->groupBoxComputedTasks->setDisabled(false);
    ui->groupBoxTaskability->setDisabled(false);

    ui->pushButtonInitialiaze->setDisabled(true);

}



void qtMightability::on_pushButtonInitialiaze_clicked()
{
    cout << "begin init" << endl;

    initMightabilities();
    initGui();

    cout << "end init" << endl;
}


void qtMightability::on_checkBox_toggled(bool checked)
{
    if (checked)
    {

        cout << "##########################################" << endl;
        cout << "Show taskabilities" << endl;
        cout << "##########################################" << endl;

        curr_params_for_show_taskability.show_all_taskability_graph=1;

        curr_params_for_show_taskability.show_taskability_node=0;
        curr_params_for_show_taskability.show_TN_by_node_ID=0;
        curr_params_for_show_taskability.show_TN_by_agent=0;

        SHOW_TASKABILITIES=1;

        //      curr_params_for_show_taskability.MN_ID=CURRENT_MANIPULABILITY_NODE_ID_TO_SHOW;
        //      curr_params_for_show_taskability.TN_ID=CURRENT_TASKABILITY_NODE_ID_TO_SHOW;
        //      curr_params_for_show_taskability.MN_perf_ag=CURRENT_TASK_PERFORMED_BY;
        //       curr_params_for_show_taskability.MN_targ_obj=CURRENT_OBJECT_TO_MANIPULATE_INDEX;
        //       curr_params_for_show_taskability.TN_perf_ag=CURRENT_TASK_PERFORMED_BY;
        //       curr_params_for_show_taskability.TN_targ_ag=CURRENT_TASK_PERFORMED_FOR;
        //       curr_params_for_show_taskability.TN_task=CURRENT_HRI_MANIPULATION_TASK;

        //       g3d_draw_allwin_active();

    }
    else
    {
        cout << "##########################################" << endl;
        cout << "Hide taskabilities" << endl;
        cout << "##########################################" << endl;

        SHOW_TASKABILITIES = 0;
    }

}


void qtMightability::on_pushButtonTaskability_clicked()
{
    cout << "##########################################" << endl;
    cout << "Begin compute Taskalities" << endl;
    cout << "##########################################" << endl;

#ifdef USE_MIGHTABILITY_MAPS
    find_taskability_graph();
    print_taskability_graph();
#else
    cout<< "Flag USE_MIGHTABILITY_MAPS not on"<<endl;
#endif
    //    update_taskability_graph_list();

    cout << "##########################################" << endl;
    cout << "End of compute Taskalities" << endl;
    cout << "##########################################" << endl;
}

void qtMightability::on_pushButtonPlanTask_clicked()
{
    clock_t start = clock();
    // initialisation of variable
    HRI_task_desc task;

    bool byAgent = false;
    bool forAgent = false;
    bool btask = false;

    for (unsigned int vi = 0; vi < HRI_task_NAME_Enum_map.size(); vi++)
    {
        QString str = QString::fromStdString(HRI_task_NAME_Enum_map.at(vi).first);
        if (ui->comboBoxTask->currentText().compare(str) == 0 )
        {
            task.task_type = HRI_task_NAME_Enum_map.at(vi).second;
            btask = true;
        }
    }


    for (unsigned int vi = 0; vi < HRI_agent_NAME_Enum_map.size(); vi++)
    {
        QString str = QString::fromStdString(HRI_agent_NAME_Enum_map.at(vi).first);

        if (ui->comboBoxPerformer->currentText().compare(str) == 0)
        {
            task.by_agent = HRI_agent_NAME_Enum_map.at(vi).second;
            byAgent = true;
        }
        if (ui->comboBoxAgentFor->currentText().compare(str) == 0)
        {
            task.for_agent = HRI_agent_NAME_Enum_map.at(vi).second;
            forAgent = true;
        }
    }
    if(!btask || !byAgent || !forAgent)
    {
        cout << "ERROR: Somethings is not right : \ntask type = " <<  task.task_type << "\nBy agent : "
             << task.by_agent << "\nFor agent : " << task.for_agent << endl;
    }


//    task.task_type = HRI_task_NAME_Enum_map[ui->comboBoxTask->currentText().toStdString()];
//    task.by_agent = HRI_agent_NAME_Enum_map[ui->comboBoxPerformer->currentText().toStdString()];
//    task.for_agent = HRI_agent_NAME_Enum_map[ui->comboBoxAgentFor->currentText().toStdString()];


//    for( map<HRI_TASK_AGENT,string>::iterator ii=HRI_agent_NAME_Enum_map.begin(); ii!=HRI_agent_NAME_Enum_map.end(); ++ii)
//    {
//        if (QString::fromStdString((*ii).second).contains(ui->comboBoxPerformer->currentText()))
//        {
//            task.by_agent = (*ii).first;
//        }
//        else if (QString::fromStdString((*ii).second).contains(ui->comboBoxAgentFor->currentText()))
//        {
//            task.for_agent = (*ii).first;
//        }
//    }

    task.for_object = ui->comboBoxObject->currentText().toStdString();


    CURRENT_HRI_MANIPULATION_TASK = task.task_type;
    CURRENT_TASK_PERFORMED_BY = task.by_agent;
    CURRENT_TASK_PERFORMED_FOR = task.for_agent;
    strcpy(CURRENT_OBJECT_TO_MANIPULATE,task.for_object.c_str());


    static int task_plan_id=0;
    int for_proactive_info=0;//TASK_IS_FOR_PROACTIVE_BEHAVIOR;
    TASK_IS_FOR_PROACTIVE_BEHAVIOR = for_proactive_info;
    double try_for_seconds=60;//try planning for this much time in s

    // call for the principal functions
    if (ENV.getBool(Env::printMightabilities))
        printf("inside CB_find_current_task_solution_obj() CURRENT_OBJECT_TO_MANIPULATE= %s,  task.for_object =%s\n",CURRENT_OBJECT_TO_MANIPULATE,task.for_object.c_str());
    int val_res=validate_HRI_task(task, task_plan_id,for_proactive_info, try_for_seconds);
    if (ENV.getBool(Env::printMightabilities))
        printf("After validate_HRI_task, val_res=%d\n",val_res);
    task_plan_id++;
//    update_HRI_task_plan_list();
//    if (ENV.getBool(Env::printMightabilities))
//        printf("After update_HRI_task_plan_list \n");

//    fl_check_forms();
//    g3d_draw_allwin_active();
    m_mainWindow->drawAllWinActive();


    if(val_res==0)
    {//TODO: This
        //backtrack_for_this_task_feasibility(task, task_plan_id,for_proactive_info, try_for_seconds);
    }

    if (ENV.getBool(Env::printMightabilities))
        printf("returning from CB_find_current_task_solution_obj()\n");
    printf("Time elapsed: %f\n", ((double)clock() - start) / CLOCKS_PER_SEC);
    return;
}

void qtMightability::on_pushButtonExecuteAll_clicked()
{
    show_desired_HRI_task_plan();
}
