#include "qtPlaceCamera.h"
#include "ui_qtPlaceCamera.h"
#include <QtGui>




#include "qtMainInterface/mainwindowGenerated.hpp"
#include "Graphic-pkg.h"
#include "P3d-pkg.h"
#include <fstream>

extern G3D_Window *G3D_WIN;

qtPlaceCamera::qtPlaceCamera(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::qtPlaceCamera)
{
    ui->setupUi(this);
    this->setWindowTitle("Camera Placement");

    pp3d_env e = (pp3d_env)p3d_get_desc_curid(P3D_ENV);
    pp3d_box b = &e->box;

    ui->doubleSpinBoxx->setMinimum(b->x1);
    ui->doubleSpinBoxy->setMinimum(b->y1);
    ui->doubleSpinBoxz->setMinimum(b->z1);

    ui->doubleSpinBoxx->setMaximum(b->x2);
    ui->doubleSpinBoxy->setMaximum(b->y2);
    ui->doubleSpinBoxz->setMaximum(b->z2);

    initAll();
}

qtPlaceCamera::~qtPlaceCamera()
{
    delete ui;
}

void qtPlaceCamera::initAll()
{
    g3d_states vs = g3d_get_cur_states();

    QtShiva::SpinBoxSliderConnector* Sx = new QtShiva::SpinBoxSliderConnector(this, ui->doubleSpinBoxx,	ui->horizontalSliderx);
    Sx->setValue(vs.x);
    QtShiva::SpinBoxSliderConnector* Sy = new QtShiva::SpinBoxSliderConnector(this, ui->doubleSpinBoxy,	ui->horizontalSlidery);
    Sy->setValue(vs.y);
    QtShiva::SpinBoxSliderConnector* Sz = new QtShiva::SpinBoxSliderConnector(this, ui->doubleSpinBoxz,	ui->horizontalSliderz);
    Sz->setValue(vs.z);

    QtShiva::SpinBoxSliderConnector* SDist = new QtShiva::SpinBoxSliderConnector(this, ui->doubleSpinBoxdist,	ui->horizontalSliderdist);
    SDist->setValue(vs.zo);
    QtShiva::SpinBoxSliderConnector* SAz = new QtShiva::SpinBoxSliderConnector(this, ui->doubleSpinBoxaz,	ui->horizontalSlideraz);
    SAz->setValue(vs.az);
    QtShiva::SpinBoxSliderConnector* SEl = new QtShiva::SpinBoxSliderConnector(this, ui->doubleSpinBoxel,	ui->horizontalSliderel);
    SEl->setValue(vs.el);

    QtShiva::SpinBoxSliderConnector* SUp0 = new QtShiva::SpinBoxSliderConnector(this, ui->doubleSpinBoxup0,	ui->horizontalSliderup0);
    SUp0->setValue(vs.up[0]);
    QtShiva::SpinBoxSliderConnector* SUp1 = new QtShiva::SpinBoxSliderConnector(this, ui->doubleSpinBoxup1,	ui->horizontalSliderup1);
    SUp1->setValue(vs.up[1]);
    QtShiva::SpinBoxSliderConnector* SUp2= new QtShiva::SpinBoxSliderConnector(this, ui->doubleSpinBoxup2,	ui->horizontalSliderup2);
    SUp2->setValue(vs.up[2]);
    QtShiva::SpinBoxSliderConnector* SUp3 = new QtShiva::SpinBoxSliderConnector(this, ui->doubleSpinBoxup3,	ui->horizontalSliderup3);
    SUp3->setValue(vs.up[3]);
}

void qtPlaceCamera::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void qtPlaceCamera::on_pushButtonSave_clicked()
{
    char* home_path = getenv("HOME_MOVE3D");
    std::string path("~/");
    if( home_path == NULL) {
      std::cout << "HOME_MOVE3D is not defined" << std::endl;
    }
    else
    {
        path = home_path;
    }

    QString fileName = QFileDialog::getSaveFileName(this,
                                tr("Save to"),
                                path.c_str(),
                                tr("Camera file (*.p3dCam)"));
    if (!fileName.isEmpty())
        printf("filename : %s", fileName.toUtf8().constData());

    std::ofstream ofs;
    ofs.open (fileName.toUtf8().constData(), std::ofstream::out | std::ofstream::trunc);

    g3d_states vs = g3d_get_cur_states();

    ofs<<"vs.x = "<<vs.x<<std::endl;
    ofs<<"vs.y = "<<vs.y<<std::endl;
    ofs<<"vs.z = "<<vs.z<<std::endl;

    ofs<<"vs.zo = "<<vs.zo<<std::endl;
    ofs<<"vs.az = "<<vs.az<<std::endl;
    ofs<<"vs.el = "<<vs.el<<std::endl;

    ofs<<"vs.up[0] = "<<vs.up[0]<<std::endl;
    ofs<<"vs.up[1] = "<<vs.up[1]<<std::endl;
    ofs<<"vs.up[2] = "<<vs.up[2]<<std::endl;
    ofs<<"vs.up[3] = "<<vs.up[3]<<std::endl;

    ofs.close();



}

void qtPlaceCamera::on_pushButtonLoad_clicked()
{
    char* home_path = getenv("HOME_MOVE3D");
    std::string path("~/");
    if( home_path == NULL) {
      std::cout << "HOME_MOVE3D is not defined" << std::endl;
    }
    else
    {
        path = home_path;
    }

    QString fileName = QFileDialog::getOpenFileName(this,
                                tr("Load"),
                                path.c_str(),
                                tr("Camera file (*.p3dCam)"));
    if (!fileName.isEmpty())
        printf("filename : %s \n", fileName.toUtf8().constData());


    std::ifstream input(fileName.toUtf8().constData());


    std::string line;

    double number;
    std::vector<double> numbers;
    while(getline( input, line ))
    {
        std::istringstream iss(line);
        iss.ignore(256,'=');
        while(iss >> number)
        {
            numbers.push_back(number);
        }
    }

    G3D_WIN->vs.x = numbers[0];
    G3D_WIN->vs.y = numbers[1];
    G3D_WIN->vs.z = numbers[2];

    G3D_WIN->vs.zo = numbers[3];
    G3D_WIN->vs.az = numbers[4];
    G3D_WIN->vs.el = numbers[5];

    G3D_WIN->vs.up[0] = numbers[6];
    G3D_WIN->vs.up[1] = numbers[7];
    G3D_WIN->vs.up[2] = numbers[8];
    G3D_WIN->vs.up[3] = numbers[9];

    emit valueChanged();
    initAll();
}

void qtPlaceCamera::on_pushButtonQuit_clicked()
{
    this->close();
}


//###################### x, y, z ##########################
void qtPlaceCamera::on_horizontalSliderx_sliderMoved(int position)
{
    on_doubleSpinBoxx_valueChanged(ui->doubleSpinBoxx->value());
}

void qtPlaceCamera::on_doubleSpinBoxx_valueChanged(double v)
{
    G3D_WIN->vs.x= v;
    emit valueChanged();
}

void qtPlaceCamera::on_horizontalSlidery_sliderMoved(int position)
{
    on_doubleSpinBoxy_valueChanged(ui->doubleSpinBoxy->value());
}

void qtPlaceCamera::on_doubleSpinBoxy_valueChanged(double v)
{
    G3D_WIN->vs.y= v;
    emit valueChanged();
}

void qtPlaceCamera::on_horizontalSliderz_sliderMoved(int position)
{
    on_doubleSpinBoxz_valueChanged(ui->doubleSpinBoxz->value());
}

void qtPlaceCamera::on_doubleSpinBoxz_valueChanged(double v)
{
    G3D_WIN->vs.z= v;
    emit valueChanged();
}


//###################### zo, az, el ##########################
void qtPlaceCamera::on_horizontalSliderdist_sliderMoved(int position)
{
    on_doubleSpinBoxdist_valueChanged(ui->doubleSpinBoxdist->value());
}

void qtPlaceCamera::on_doubleSpinBoxdist_valueChanged(double v)
{
    G3D_WIN->vs.zo= v;
    emit valueChanged();
}

void qtPlaceCamera::on_horizontalSlideraz_sliderMoved(int position)
{
    on_doubleSpinBoxaz_valueChanged(ui->doubleSpinBoxaz->value());
}

void qtPlaceCamera::on_doubleSpinBoxaz_valueChanged(double v)
{
    G3D_WIN->vs.az= v;
    emit valueChanged();
}

void qtPlaceCamera::on_horizontalSliderel_sliderMoved(int position)
{
    on_doubleSpinBoxel_valueChanged(ui->doubleSpinBoxel->value());
}

void qtPlaceCamera::on_doubleSpinBoxel_valueChanged(double v)
{
    G3D_WIN->vs.el= v;
    emit valueChanged();
}

//###################### up ##########################
void qtPlaceCamera::on_horizontalSliderup0_sliderMoved(int position)
{
    on_doubleSpinBoxup0_valueChanged(ui->doubleSpinBoxup0->value());
}

void qtPlaceCamera::on_doubleSpinBoxup0_valueChanged(double v)
{
    G3D_WIN->vs.up[0]= v;
    emit valueChanged();
}

void qtPlaceCamera::on_horizontalSliderup1_sliderMoved(int position)
{
    on_doubleSpinBoxup1_valueChanged(ui->doubleSpinBoxup1->value());
}

void qtPlaceCamera::on_doubleSpinBoxup1_valueChanged(double v)
{
    G3D_WIN->vs.up[1]= v;
    emit valueChanged();
}

void qtPlaceCamera::on_horizontalSliderup2_sliderMoved(int position)
{
    on_doubleSpinBoxup2_valueChanged(ui->doubleSpinBoxup2->value());
}

void qtPlaceCamera::on_doubleSpinBoxup2_valueChanged(double v)
{
    G3D_WIN->vs.up[2]= v;
    emit valueChanged();
}

void qtPlaceCamera::on_horizontalSliderup3_sliderMoved(int position)
{
    on_doubleSpinBoxup3_valueChanged(ui->doubleSpinBoxup3->value());
}

void qtPlaceCamera::on_doubleSpinBoxup3_valueChanged(double v)
{
    G3D_WIN->vs.up[3]= v;
    emit valueChanged();
}








